#encoding=utf-8
from utils import setlogging,getpathinfo
import os,sys
from utils import timeoperation
logsdir =  getpathinfo.get_dir(os.path.abspath(__file__),'logs/')
setlogging.set_logging(logsdir+r'log-master.txt')

path = os.path.abspath(__file__)
absdir = path[0:path.rfind('/')]
sys.path.append(absdir[0:absdir.rfind('/',0,len(absdir))])

from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client
from utils import configoperations,clean,collectinfo
import sys
reload(sys)
sys.setdefaultencoding('utf8')
from common.const import type
import workerpool
import taskpool
from time import sleep
from common.sdk import s3client, tencentclient, aliyunclient, clientfactory
import processpool
import receivemessage
import handlefatalerror
import monitor
import logging
log = None

class Master:
    def __init__(self,configFileName,absdir,jobsdir):
        self.configFileName = configFileName 
        self.absdir = absdir
        self.read_configfile()
        self.check_configfile()
        self.thisjobdir = jobsdir + self.job['job-ID']+'/'
        self.reportFile = collectinfo.generate_report(self.thisjobdir,self.job['job-ID']+'_report',self.job)
        
    def read_configfile(self):
        try:
            configFile = open(self.absdir+self.configFileName,"r")
            self.job = dict()
            self.workerlist = list()
            if not configoperations.read_config(configFile,self.job,self.workerlist):
                exit(1)
            configFile.close()
        except IOError,e:
            log.error(e)
            exit(1)
            
    def check_configfile(self):
        if not configoperations.check_config_and_fillin_default(self.job,self.workerlist):
            exit(1)
            
    def do_synctask(self):
        self.taskPool.init_sync_info(self.job)
        if self.job['job-type'] == type.JobType.Transfer:
            self.dispatcher(type.JobType.Transfer)
            self.generate_transfer_errorlist()
            self.clean_transfer()
            self.write_synctask_status(type.JobType.Transfer, 'done')
     
        if self.job['job-type'] == type.JobType.Check or self.job['check-time'] == 'now':
            self.dispatcher(type.JobType.Check)
            self.generate_check_errorlist()
            self.generate_md5_list()
            self.clean_check()
            self.write_synctask_status(type.JobType.Check, 'done')
            
        self.reportFile.close()
        self.clean_syntask()
        if self.job['sync-enable-increment']=='True':
            self.write_synctask_status('', 'done')
            
    def dispatcher(self,jobType):
        self.taskPool.generate_task_from_job_thread(jobType,self.desClient)
        receivemessage.init()
        #isTaskstatusError是指自己内部出现错误，如果为True,直接进入下一轮
        isInternalTaskstatusError = False
        while True:
            if self.is_this_round_finished(isInternalTaskstatusError):
                isInternalTaskstatusError = False
                [requireRoundNum,startRound,currentRound] = self.taskPool.get_round_info()
                collectinfo.write_report_roundInfo(self.reportFile,self.job['synctask'],jobType,currentRound,self.taskPool.get_all_files_size(),receivemessage.get_round_finishedInfo())
                self.currentRound = currentRound
                if not (taskpool.put_roundstatus(self.desClient, self.job['des-bucketName'],
                    self.job['synctask']+'/'+jobType+'/round'+str(currentRound)+'/roundstatus', 'done')):
                    exit(1)
                if (currentRound+1-startRound == requireRoundNum):
                    log.info('all round finished')
                    break
                else:
                    receivemessage.init()
                    self.taskPool.init()
                    log.info('round '+str(currentRound)+' of '+self.job['synctask']+' has finished')
                    self.taskPool.generate_nextround_task(self.currentRound+1)
            taskInfo = self.taskPool.get()
            if not taskInfo:
                isInternalTaskstatusError = handlefatalerror.is_taskstatus_error(self.taskPool.is_generating_tasks(), \
                                                                             self.workerPool.is_workers_all_ready())
                sleep(5)
            if taskInfo:
                worker = self.workerPool.get()
                if worker:
                    self.processPool.generate_a_process(taskInfo, worker, self.desClient)
                else:
                    log.error('all workers are all off!')
                    exit(1)
        
    def write_synctask_status(self, jobType,status):
        bucketName = self.job['des-bucketName']
        if jobType == '':
            keyName = self.job['synctask'] + '/syncstatus'
        else: 
            keyName = self.job['synctask'] + '/' + jobType+'/'+jobType+'status'
        if not self.desClient.put_object(bucketName, keyName, status):
            return False
        else:
            log.info('write '+keyName +':'+status)
            return True
        
    def is_this_round_finished(self,isTaskstatusError):
        return ((self.taskPool.is_all_tasks_finished() and self.taskPool.is_generating_tasks()==False)) or (isTaskstatusError == True)
    def generate_transfer_errorlist(self):
        collectinfo.generate_errorlist(self.desClient, self.job['des-bucketName'],type.JobType.Transfer,self.job['synctask'],\
                                             self.currentRound,self.thisjobdir+self.job['transfer-error-output'])
        
    def clean_transfer(self):
        clean.clean_synctask(self.desClient,self.job['des-bucketName'], type.JobType.Transfer, self.job['synctask'],self.is_clean_transferRound0())
        
    def clean_syntask(self):
        if self.job['sync-enable-increment']=='True' or (self.job['job-type'] == type.JobType.Transfer and self.job['check-time'] == 'future'):
            pass
        else:
            clean.clean_synctask(self.desClient,self.job['des-bucketName'], '', self.job['synctask'],True)
            
    def is_clean_transferRound0(self):
        if self.job['check-mode'] == 'never':
            return True
        else:
            return False
        
    def generate_check_errorlist(self):
        collectinfo.generate_errorlist(self.desClient, self.job['des-bucketName'],type.JobType.Check,self.job['synctask'],\
                                             self.currentRound,self.thisjobdir+self.job['check-error-output'])
        
    def generate_md5_list(self):
        if self.job['check-mode'] == 'md5':
            return collectinfo.generate_md5list(self.desClient, self.job['des-bucketName'],self.job['synctask'],self.thisjobdir+self.job['check-md5-output'])
        else:
            return -1
        
    def clean_check(self):
        clean.clean_synctask(self.desClient,self.job['des-bucketName'],type.JobType.Check, self.job['synctask'],True)
        
    def create_desClient(self):
        self.desClient = s3client.S3Client(self.job['des-accesskey'],self.job['des-secretkey'],self.job['des-endpoint'])
        receivemessage.ReceiveMessage.desClient = self.desClient
        
    def init(self):
        self.create_desClient()
        self.workerPool = workerpool.WorkerPool() 
        for worker in self.workerlist:
            self.workerPool.put(worker)
        self.taskPool = taskpool.TaskPool(self.workerPool)
        self.processPool = processpool.ProcessPool(self.taskPool,self.workerPool)
        self.monitor = monitor.Monitor(self.processPool,self.workerPool,self.taskPool)
        receivemessage.create_rpc_server(self.job['master'],self.taskPool,self.workerPool,self.processPool,self.monitor)
        if not self.isCreateReceiveMessageServerOK():
            log.error('create the simplexmlrpc server failed')
            exit(1)
            
    def isCreateReceiveMessageServerOK(self):
        while True:
            if receivemessage.ReceiveMessage.isCreateReceiveMessageServerOK == 0:
                return False
            elif receivemessage.ReceiveMessage.isCreateReceiveMessageServerOK == 1:
                return True
            elif receivemessage.ReceiveMessage.isCreateReceiveMessageServerOK == -1:
                pass
            sleep(1)
            
    def build_synctask_context(self):
        
        if self.job['sync-enable-increment'] == 'False':
            clean.clean_job(self.desClient, self.job['des-bucketName'], self.job['job-ID'])
        synctaskCnt = 0
        while self.desClient.does_object_exists(self.job['des-bucketName'],
                                                self.job['job-ID']+'/synctask'+str(synctaskCnt)+'/syncstatus'):
            synctaskCnt+=1
            
        if synctaskCnt == 0:
            self.create_synctask(synctaskCnt, False)
            log.info('create synctask0 successfully!')
        else:
            lines = self.desClient.get_object_with_stream(self.job['des-bucketName'],
                                                self.job['job-ID']+'/synctask'+str(synctaskCnt-1)+'/syncstatus')
            if not lines:
                exit(1)
            line = lines.readline()
            if line == 'done':
                self.create_synctask(synctaskCnt, False)
                log.info('create synctask'+str(synctaskCnt)+' successfully!')
            else:
                self.create_synctask(synctaskCnt-1, True)
                log.info('recover synctask'+str(synctaskCnt-1)+' successfully!')
        return True
    
    def get_source_time(self):
        if self.job['src-filetype'] == type.FileType.DiskFile:
            serverTime = timeoperation.get_filesystem_time(self.job['src-absolutepath'],self.job['sync-enable-increment'])
        else:
            client = clientfactory.create_client(self.job['src-filetype'],self.job['src-accesskey'],self.job['src-secretkey'],self.job['src-endpoint'])
            serverTime = timeoperation.get_server_time(client,self.job['src-bucketName'], self.job['synctask'])
        if serverTime == -1:
            exit(1)
        return serverTime
    
    def create_synctask(self, synctaskCnt, isRecoverThisSyncTask):
        self.job['synctask'] = self.job['job-ID']+'/synctask'+str(synctaskCnt)
        if synctaskCnt == 0:
            if isRecoverThisSyncTask == False:
                self.create_new_synctask()
            else:
                self.recover_old_synctask()
        else:
            lines = self.desClient.get_object_with_stream(self.job['des-bucketName'],
                                                self.job['job-ID']+'/synctask'+str(synctaskCnt-1)+'/starttime')
            if not lines:
                exit(1)
            self.job['sync-lastmodify'] = str(float(lines.readline())-300)
            if isRecoverThisSyncTask ==False:
                self.create_new_synctask()
            else:
                self.recover_old_synctask()
    
    def create_new_synctask(self):
        if self.job['sync-enable-increment']=='False':
            serverTime = 1.7e+308
        else:
            serverTime = self.get_source_time()
        if serverTime == -1:
            log.error('get server time failed')
            exit(1)
        if not self.desClient.put_object(self.job['des-bucketName'],self.job['synctask']+'/starttime',str(serverTime)):
            log.error('put server time failed')
            exit(1)
        if not self.desClient.put_object(self.job['des-bucketName'],self.job['synctask']+'/nextmarker',''):
            log.error('put syncstatus of '+self.job['synctask']+' failed')
            exit(1)
        if not self.desClient.put_object(self.job['des-bucketName'],self.job['synctask']+'/syncstatus','running'):
            log.error('put syncstatus of '+self.job['synctask']+' failed')
            exit(1)
        self.job['nextmarker']=''
        self.job['sync-starttime'] = serverTime
        
    def recover_old_synctask(self):
        lines = self.desClient.get_object_with_stream(self.job['des-bucketName'],self.job['synctask']+'/starttime')
        if not lines:
            exit(1)
        self.job['sync-starttime'] = lines.readline()
        lines = self.desClient.get_object_with_stream(self.job['des-bucketName'],self.job['synctask']+'/nextmarker')
        if not lines:
            exit(1)
        self.job['nextmarker'] = lines.readline()
                
        
if __name__ == '__main__':
    absdir = getpathinfo.get_absdir(os.path.abspath(__file__))
    logsdir =  getpathinfo.get_dir(os.path.abspath(__file__),'logs/')
    jobsdir = getpathinfo.get_dir(os.path.abspath(__file__),'jobs/')
    setlogging.set_logging(logsdir+r'log-master.txt')
    log = logging.getLogger('root')
    configFileName = 'config-master'
    try:
        master = Master(configFileName,absdir,jobsdir)
        master.init()
        master.build_synctask_context()
        master.do_synctask()
        
    except (SystemExit,KeyboardInterrupt):
        """异常退出,不删除临时文件，直接退出"""
        log.error('something unexpected happaned,you can restart master to continue the unfinished job!')
    finally:
        log.info('the master is closed')
        






