#encoding=utf-8
import os,sys
path = os.path.abspath(__file__)
absdir = path[0:path.rfind('/')]
sys.path.append(absdir[0:absdir.rfind('/',0,len(absdir))])

import xmlrpclib
import threading
from common.sdk import s3client,qiniuclient,aliyunclient,tencentclient
from common.enum import status
from collections import deque
from utils import clean, generateID
from utils import clean, generateID, readdata
from common.const import type
import time
import logging

log = logging.getLogger('root')

class TaskPool:

    lockTasklist = threading.Lock()
    lockFailedtasklist = threading.Lock()
    def __init__(self,workerPool):
        self.workerPool = workerPool
        
    def init(self):
        #初始化
        self.taskNumbers = 0
        self.tasklistStartPosToFind = 0
        self.tasklist = deque()
        self.failedtasklist = deque()
        #打印使用,all指的是一个job
        self.allFiles = 0
        self.allSize = 0
        #设置一个正在生成tasks的标志，一轮完成的判断条件
        self.isGeneratingTasks = False
        
    def put(self,task):
        '''
        :param task:这里有点矛盾哦，task在这里其实是一个list,task[0]是真正的task,task[1]是该task的状态，状态如下：
                    0:ready(未调度)，1:dispatched(已被调度，正在迁移）,2:done(完成),3:failed(失败已经放入failedtasklist)
        '''
        self.tasklist.append(task)
        self.taskNumbers_pp()

    def put_failed_task(self,task):
        '''
        :param task:这里有点矛盾哦，task在这里其实是一个list,task[0]是真正的task,task[1]是该task的状态，状态如下：
                    错误task列表里是有一个状态的,0:ready(未调度)，1:dispatched(已被调度，正在迁移）,2:done(完成)
        '''
        TaskPool.lockFailedtasklist.acquire()
        self.failedtasklist.append(task)
        pos = len(self.failedtasklist)-1
        TaskPool.lockFailedtasklist.release()
        return pos 
    def set_task_status(self,pos,status,oldstatus=-1):
        '''
        该函数用来设置tasklist中某个task的状态
        :param pos: 要设置的task在tasklist中的位置
        :param status:将该task设置为status状态
        :param oldstatus:如果oldstatus==-1,可直接设置task的状态为status;
                         否则，只有当task的状态是oldstatus时，才可以设置
               设置状态的函数的逻辑和这个函数基本类似
        '''
        TaskPool.lockTasklist.acquire()
        if oldstatus != -1:
            if self.tasklist[pos][1]!=oldstatus:
                TaskPool.lockTasklist.release()
                return False
        self.tasklist[pos][1] = status
        TaskPool.lockTasklist.release()
        return True
    def set_failedtask_status(self,pos = -1,status=0,oldstatus = -1):
        '''
        :param pos: 要设置的task在tasklist中的位置，如果为-1，就使用keyForTaskPath来遍历查找
        :param status:将该task设置为status状态
        :param oldstatus:如果oldstatus==-1,可直接设置task的状态为status;
                         否则，只有当task的状态是oldstatus时，才可以设置
        '''
        TaskPool.lockFailedtasklist.acquire()
        if oldstatus != -1:
            if self.failedtasklist[pos][1] != oldstatus:
                TaskPool.lockFailedtasklist.release()
                return False
        self.failedtasklist[pos][1] = status
        TaskPool.lockFailedtasklist.release()
        return True
    def get_task_status(self,pos):
        '''
        获取tasklist中位于pos位置task的状态
        '''
        if pos>=0 and pos < len(self.tasklist):
            return self.tasklist[pos][1]
        return -1
    def get(self):
        '''
        get用来取出一个可以调度的task,get的逻辑如下：
        先在tasklist中查找状态为0的task,从tasklistStartPosToFind的位置开始遍历；
        如果在tasklist中没有找到可调度的task,再到failedtasklist中查找可调度的；
        failedtasklist的查找逻辑是从头开始遍历，如果仍未找到，返回None
        '''
        '''
        :return：返回值是一个list,我们称为taskInfo,taskInfo[0]是task；
                 taskInfo[1] 是该task的ID,生成方式参考下面的代码
        '''
        while self.tasklistStartPosToFind<len(self.tasklist):
            if self.tasklist[self.tasklistStartPosToFind][1] == status.TASK.ready:
                if self.set_task_status(self.tasklistStartPosToFind, status.TASK.dispatched, status.TASK.ready):
                    task = (self.tasklist[self.tasklistStartPosToFind])[0]
                    self.tasklistStartPosToFind += 1
                    #self.tasklistStartPosToFind-1 是该task在tasklist中的位置
                    #True 标识该task是从tasklist中取出的，如果从failedtasklist中取出，标识为false
                    taskID = generateID.taskID(str(time.time())+' '+self.destination['bucketName']+' '+str(self.tasklistStartPosToFind-1)+' '+'True')
                    return [task,taskID]
            self.tasklistStartPosToFind += 1
        return self.get_from_failed_tasklist()
    def get_from_failed_tasklist(self):
        """从头开始查找可调度的task"""
        for i in range(len(self.failedtasklist)):
            if self.failedtasklist[i][1]==status.TASK.ready:
                if self.set_failedtask_status(i,status.TASK.dispatched,status.TASK.ready):
                    task = (self.failedtasklist[i])[0]
                    taskID = generateID.taskID(str(time.time())+' '+self.destination['bucketName']+' '+str(i)+' '+'False')
                    return [task,taskID]
                else:
                    continue
        return None
    
    def is_all_tasks_finished(self):
        '''
        使用self.taskNumbers、tasklist和failedtasklist去判断该轮（round）是否完成
        '''
        for failedtask in self.failedtasklist:
            if failedtask[1] != status.TASK.done and failedtask[1] != status.TASK.failed:
                return False
        finished = len(self.failedtasklist)
        for task in self.tasklist:
            if task[1] == status.TASK.done:
                finished += 1
        if finished == self.taskNumbers:
            return True
        else:
            return False
    def set_generating_tasks(self,mybool):
        self.isGeneratingTasks = mybool
    def is_generating_tasks(self):
        return self.isGeneratingTasks
    def empty(self):
        if len(self.tasklist) == 0:
            return True;
        else:
            return None
    def taskNumbers_pp(self):
        self.taskNumbers+=1
    
    def get_taskNumbers(self):
        return self.taskNumbers
    
    def get_round_info(self):
        '''
        :return : 返回值是个list，分别是：config-master中配置的轮数、开始的轮数、当前正在运行的轮数
        '''
        return [self.requireRoundNum,self.startRound,self.currentRound]
    def set_current_round(self,currentRound):
        self.currentRound = currentRound
    def generate_task_from_job_thread(self,jobType,desClient):
        self.init()
        #round info
        self.startRound = 0
        self.currentRound = 0
        self.jobType = jobType
        self.isGeneratingTasks = True
        self.desClient = desClient
        self.th = threading.Thread(target=self.generate_task_from_job, args=())
        self.th.setDaemon(True)
        self.th.start()
    def init_sync_info(self,job):
        #job
        self.jobID = job['job-ID']
        self.master = job['master']
        #src
        self.source = dict()
        self.source['file-list'] = job['src-file-list']
        self.source['filetype'] = job['src-filetype']
        filetype = job['src-filetype']
        if filetype == type.FileType.DiskFile:
            self.source['absolutepath'] = job['src-absolutepath']
        else:
            self.source['accessKey'] = job['src-accesskey']
            self.source['secretKey'] = job['src-secretkey']
            self.source['endpoint'] = job['src-endpoint']
            self.source['bucketName'] = job['src-bucketName']
            self.source['key'] = job['src-key']
        #des
        self.destination = dict()
        self.destination['accessKey'] = job['des-accesskey']
        self.destination['secretKey'] = job['des-secretkey']
        self.destination['endpoint'] = job['des-endpoint']
        self.destination['bucketName'] = job['des-bucketName']
        self.destination['key'] = job['des-key']
        
        self.taskSize = long(job['task-size'])*1024*1024
        self.taskFileNumbers = long(job['task-filenumbers'])
        self.requireRoundNum = long(job['round'])
        #sync
        self.sync = dict()
        self.sync['lastmodify'] = job['sync-lastmodify']
        self.sync['starttime'] = job['sync-starttime']
        self.nextmarker = job['nextmarker']
        self.synctask = job['synctask']
        #transfer
        self.transfer = dict()
        self.transfer['is-continue'] = job['transfer-is-continue']
        #check
        self.check = dict()
        self.check['mode'] = job['check-mode']
        self.check['is-continue'] = job['check-is-continue']
        
    def generate_task_from_job(self):
        if self.source['filetype'] == type.FileType.S3File or self.source['filetype'] == type.FileType.BaiduFile:
            self.srcClient = s3client.S3Client(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.source['filetype'] == type.FileType.QiniuFile:
            self.srcClient = qiniuclient.QiniuClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.source['filetype'] == type.FileType.AliyunFile:
            self.srcClient = aliyunclient.AliyunClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.source['filetype'] == type.FileType.TencentFile:
            self.srcClient = tencentclient.TencentClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])    
        desBucketName = self.destination['bucketName']
    
        if not self.is_continue_unfinished_job():
            if not clean.clean_synctask(self.desClient,desBucketName,self.jobType,self.synctask,True):
                self.isGeneratingTasks = False
                return False
            else:
                return self.after_generate_task(self.generate_round0_task(), desBucketName)
        
        if self.is_this_job_has_finished(self.jobType):
            log.info('we have read the status of '+ self.synctask+'-'+self.jobType+' in the '+desBucketName+',and find it finished')
            self.currentRound = self.startRound+self.requireRoundNum-1
            self.isGeneratingTasks = False
            return True
        
        roundStatus = ''
        cntRound = 0
        while True: 
            roundStatus = self.get_this_round_status(desBucketName,self.synctask+'/'+self.jobType+'/round'+str(cntRound)+'/roundstatus')
            if roundStatus and roundStatus == 'done':
                cntRound += 1
            else:
                break
        if roundStatus and roundStatus == 'ready':
            return self.after_generate_task(self.get_this_round_task(cntRound), desBucketName)
        if cntRound == 0:
            log.info('recovering the unfinished '+self.synctask+' failed,we will generate round0 tasks')
            return self.after_generate_task(self.generate_round0_task(), desBucketName)
        else:
            self.startRound = cntRound
            return self.after_generate_task(self.generate_nextround_task(cntRound), desBucketName)
        
    def is_this_job_has_finished(self,jobType):
        lines = readdata.readlines(self.desClient, self.destination['bucketName'],self.synctask+'/'+jobType+'/'+jobType+'status')
        if not lines:
            return False
        else:
            line = lines.readline()
            if line == 'done':
                return True
            else:
                return False
        
    def after_generate_task(self,isGenerateTaskSuccess,desBucketName):
        self.isGeneratingTasks = False
        if isGenerateTaskSuccess:
            if put_roundstatus(self.desClient,desBucketName,self.synctask+'/'+self.jobType+'/round'+str(self.startRound)+'/roundstatus', 'ready'):
                return True
        return False
            
    def is_continue_unfinished_job(self):
        if self.jobType == type.JobType.Transfer:
            return (self.transfer['is-continue']) == "True"
        else:
            return (self.check['is-continue']) == "True"
    def get_this_round_status(self,bucketName,keyName):
        lines = readdata.readlines(self.desClient, bucketName, keyName)
        if not lines:
            return None
        line = (lines.readline()).strip('\n').strip()
        return line
        
    def get_this_round_task(self,cntRound):
        self.currentRound = cntRound
        self.startRound = cntRound
        #从本轮中获取已完成的task和未完成的task
        log.info( 'start to recover the round'+str(self.currentRound)+' for '+ self.jobType)
        taskDirs = self.desClient.get_alldir(self.destination['bucketName'], self.synctask+'/'+self.jobType+'/round'+str(self.startRound)+'/','')
        for taskdir in taskDirs:
            pair = taskdir.split('/')
            taskNumber = pair[3].strip()
            lines = readdata.readlines(self.desClient, self.destination['bucketName'], taskdir+'taskstatus')
            if not lines:
                return False
            line = (lines.readline()).strip('\n').strip()
            if line == 'done':
                log.info( taskNumber + ',finished already')
            else:
                if not reset_taskinfo(self.desClient, self.destination['bucketName'], taskdir):
                    return False
                lines = readdata.readlines(self.desClient, self.destination['bucketName'], taskdir+'filelist')
                if not lines:
                    return False
                line = (lines.readline()).strip().strip()
                pair = line.split('\t')
                self.allFiles+=long(pair[0].strip())
                self.allSize+=long(pair[1].strip())
                task = [self.synctask,self.jobType,self.source,self.destination,taskdir,self.master,self.check['mode'],self.sync]
                self.put([task,status.TASK.ready])
                log.info( str(taskNumber)+',fileNumbers:'+str(pair[0].strip())+',taskSize:'+str(pair[1].strip()))
        log.info( 'recover the round'+str(self.currentRound)+' successfully')
        return True
    
    def generate_nextround_task(self,cntRound):
        self.currentRound = cntRound
        log.info( 'start to generate tasks(round '+str(self.currentRound)+') for '+self.jobType)
        taskNumber = 0
        taskPrefix = self.synctask+'/'+self.jobType+'/round'+str(self.currentRound)+'/task'
        currentSize = 0
        currentFileNumbers = 0
        filelist = list()
        oldTaskDirs = self.desClient.get_alldir(self.destination['bucketName'], self.synctask+'/'+self.jobType+'/round'+str(self.currentRound-1)+'/','')
        for oldTaskDir in oldTaskDirs:
            lines = readdata.readlines(self.desClient, self.destination['bucketName'], oldTaskDir + 'taskstatus')
            if not lines:
                return False
            line = (lines.readline()).strip('\n').strip()
            fileName = None
            if line == 'done':
                fileName = 'errorlist'
            else:
                fileName = 'filelist'
            lines = readdata.readlines(self.desClient, self.destination['bucketName'],oldTaskDir + fileName)
            if not lines:
                return False
            line = lines.readline()
            line = lines.readline()
            while line:
                line = line.strip('\n')
                objectSize = line.split('\t')[1].strip()
                filelist.append(line)
                currentSize += long(objectSize)
                currentFileNumbers += 1
                if currentSize >= self.taskSize or currentFileNumbers>=self.taskFileNumbers:
                    if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                        return False
                    taskNumber += 1
                    currentFileNumbers = 0
                    currentSize = 0
                    filelist[:] = []
                line = lines.readline()
        if currentSize != 0 or currentFileNumbers != 0:
            if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                return False
        log.info( 'generate tasks successfully')
        return True
        
    def generate_round0_task(self):
        self.currentRound = 0
        self.startRound = 0
        if self.source['file-list']:
            return self.generate_round0_task_from_file()
        if self.jobType == type.JobType.Check:
            if self.generate_checktask_from_transfer_round0():
                return True
        if self.source['filetype'] == type.FileType.DiskFile:
            return self.generate_round0_task_diskfile()
        else:
            return self.generate_round0_task_s3file()
    #从用户指定的文件生成task(s)
    def generate_round0_task_from_file(self):
        log.info( 'start to generate tasks(round '+str(self.currentRound)+') for '+self.jobType+' from an appointed file')
        taskNumber = 0
        taskPrefix = self.synctask+'/'+self.jobType+'/round0/task'
        currentFileNumbers = 0
        currentSize = 0
        filelist = list()
        try:
            myfile = open(self.source['file-list'],'r')
            for line in myfile:
                line = line.strip('\n')
                objectSize = line.split('\t')[1].strip()
                filelist.append(line)
                currentSize += long(objectSize)
                currentFileNumbers += 1
                if currentSize >= self.taskSize or currentFileNumbers>=self.taskFileNumbers:
                    if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                        return False
                    taskNumber += 1
                    currentFileNumbers = 0
                    currentSize = 0
                    filelist[:] = []
            if currentSize != 0 or currentFileNumbers != 0:
                if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                    return False
            log.info( 'generate tasks successfully')
            return True
        except IOError,e:
            log.error(e)
            return False
        except Exception,e:
            log.error(e)
            return False
    #like generate_next_round
    def generate_checktask_from_transfer_round0(self):
        log.info('generate checktask from transfer round0')
        log.info( 'start to generate tasks(round '+str(self.currentRound)+') for '+self.jobType)
        taskNumber = 0
        taskPrefix = self.synctask+'/'+self.jobType+'/round0/task'
        currentSize = 0
        currentFileNumbers = 0
        filelist = list()
        transferTaskDirs = self.desClient.get_alldir(self.destination['bucketName'], self.synctask+'/'+type.JobType.Transfer+'/round0/','')

        for transferTaskDir in transferTaskDirs:
            lines = readdata.readlines(self.desClient, self.destination['bucketName'],transferTaskDir + 'filelist')
            if not lines:
                return False
            line = lines.readline()
            line = lines.readline()
            while line:
                line = line.strip('\n')
                objectSize = line.split('\t')[1].strip()
                filelist.append(line)
                currentSize += long(objectSize)
                currentFileNumbers += 1
                if currentSize >= self.taskSize  or currentFileNumbers>=self.taskFileNumbers:
                    if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                        return False
                    taskNumber += 1
                    currentFileNumbers = 0
                    currentSize = 0
                    filelist[:] = []
                line = lines.readline()
        if currentSize != 0 or currentFileNumbers != 0:
            if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                return False
        if self.allFiles == 0:
            return False
        return True
        
    def generate_round0_task_s3file(self):
        taskNumber = 0
        taskPrefix = self.synctask+'/'+self.jobType+'/round0/task'
        currentFileNumbers = 0
        currentSize = 0
        filelist = list()
        #queue is for saving prefix
        queue = list()
        queue.append(self.source['key'])
        bucketName = self.source['bucketName']
        log.info( 'start to generate '+self.jobType+'-tasks(round 0) for '+self.synctask)
        while queue:
            prefix = queue.pop(0)
            marker = ''
            while True:
                returnVal = self.srcClient.list_objects( bucketName,prefix,marker)
                if not returnVal:
                    return False
                [objects,dirs,isTruncated,marker] = returnVal
                if dirs:
                    for mydir in dirs:
                        queue.append(mydir)
                for i in range(0,len(objects)):
                    objectSize,objectLastModify = self.srcClient.get_object_size_and_lastmodify( bucketName, objects[i])
                    if self.isRightTimeToOpeareteThisFile(objectLastModify):
                        filelist.append(objects[i]+'\t'+str(objectSize))
                        currentSize+=long(objectSize)
                        currentFileNumbers += 1
                    if currentSize >=self.taskSize or currentFileNumbers>=self.taskFileNumbers:
                        if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                            return False
                        taskNumber += 1
                        currentSize = 0
                        currentFileNumbers = 0
                        filelist[:] = []
                if not isTruncated:
                    break
        if currentSize != 0 or currentFileNumbers != 0:
            if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                return False
        log.info( 'generate tasks for '+self.synctask+' successfully')
        return True
    
    def isRightTimeToOpeareteThisFile(self,lastModify):
        if lastModify == -1:
            return False
        else:
            if float(lastModify) >= float(self.sync['lastmodify']) and float(lastModify) <= float(self.sync['starttime']):
                return True
            else:
                return False
            
    def generate_round0_task_diskfile(self):
        fileNamePrefix = self.synctask+'/diskfilelist/'
        if not self.generate_diskfile_list(fileNamePrefix):
            return False
        taskNumber = 0
        taskPrefix = self.synctask+'/'+self.jobType+'/round0/task'
        currentFileNumbers = 0
        currentSize = 0
        filelist = list()
        log.info( 'start to generate '+self.jobType+'-tasks(round 0) for '+self.synctask)
        filelistCnt = 0
        while True:
            filelistName = fileNamePrefix+'filelist'+str(filelistCnt)
            lines = readdata.readlines(self.desClient, self.destination['bucketName'],filelistName)
            if not lines:
                if self.desClient.does_object_exists(self.destination['bucketName'],fileNamePrefix+'status')==False:
                    time.sleep(10)
                    continue
                else:
                    log.info('the worker generate filelist finished')
                    break
            else:
                filelistCnt+=1
                line = lines.readline()
                while(line):
                    line = line.strip().strip('\n')
                    pair = line.split('\t')
                    for i in range(0,len(pair)):
                        pair[i] = pair[i].strip()
                    filelist.append(line)
                    currentSize+=long(pair[1])
                    currentFileNumbers += 1
                    if (currentSize >= self.taskSize) or (currentFileNumbers >= self.taskFileNumbers):
                        if not self.generate_a_task(filelist,currentFileNumbers,currentSize,taskNumber,taskPrefix):
                            return False
                        taskNumber += 1
                        currentSize = 0
                        currentFileNumbers = 0
                        filelist[:] = []
                    line = lines.readline()
                
        if currentSize != 0 or currentFileNumbers != 0:
            if not self.generate_a_task(filelist, currentFileNumbers, currentSize, taskNumber, taskPrefix):
                return False
        log.info( 'generate tasks for '+self.synctask+' successfully')
        return True
    
    def generate_a_task(self,filelist,currentFileNumbers,currentSize,taskNumber,taskPrefix):
        filelist.insert(0, str(currentFileNumbers)+'\t'+str(currentSize))
        if not put_task(self.desClient, self.destination['bucketName'],taskPrefix+str(taskNumber)+'/',filelist):
            return False
        self.allFiles += currentFileNumbers
        self.allSize += currentSize
        keyForTaskPath = taskPrefix+str(taskNumber)+'/'
        task = [self.synctask,self.jobType,self.source,self.destination,keyForTaskPath,self.master,self.check['mode'],self.sync]
        self.put([task,status.TASK.ready])
        log.info( 'task'+str(taskNumber) +' '+ self.synctask +' '+ self.jobType 
                             + ' fileNumbers:'+str(currentFileNumbers)+' taskSize:'+str(currentSize))
        return True
    
    def generate_diskfile_list(self,fileNamePrefix):
        clean.clean_job(self.desClient, self.destination['bucketName'], fileNamePrefix)
        worker = self.workerPool.get()
        if not worker:
            log.error( 'pity,there is no worker avaliable!')
            log.info( "failed to create the filelist,you must check why !!!")
            return False
        try:
            log.info( 'start to appoint a worker to generate file list,this worker is:'+str(worker))
            s = xmlrpclib.ServerProxy('http://'+worker)
            returnVal = s.run_build_list(self.source['absolutepath'],self.destination['accessKey'],self.destination['secretKey'],\
                                    self.destination['endpoint'],self.destination['bucketName'],fileNamePrefix,self.sync)
            if not returnVal:
                raise Exception
            log.info( 'the worker is starting to generate filelist')
            return True
        except Exception,e:
            log.error(e)
            log.error( "failed to create the filelist")
            return False
        finally:
            self.workerPool.set_worker_status(worker)
    def get_all_files_size(self):
        return [self.allFiles,self.allSize]
        
        
def put_task(client, bucketName,prefix,fileList):
    if not (client.put_object( bucketName, prefix+'filelist','\n'.join(fileList))):
        return False
    if not (client.put_object( bucketName, prefix+'errorlist','')):
        return False
    if not (client.put_object( bucketName, prefix+'taskstatus','ready')):
        return False
    if not (client.put_object( bucketName, prefix+'md5list','')):
        return False
    return True
    
def put_roundstatus(client,bucketName,keyname,status):
    if not (client.put_object(bucketName,keyname,status)):
        return False
    return True
def reset_taskinfo(client,bucketName,keyForTaskPath):
    if not (client.put_object(bucketName,keyForTaskPath+'taskstatus','ready')):
        return False
    if not (client.put_object(bucketName,keyForTaskPath+'errorlist','')):
        return False
    if not (client.put_object( bucketName, keyForTaskPath+'md5list','')):
        return False
    return True
    
    #如果下面的workerstatus也设置了，那么当master退出再重启时，会导致worker无法读取workerstatus
    """
    if client.does_object_exists(bucketName,keyForTaskPath+'workerstatus'):
        if not (client.delete_one_object(bucketName,keyForTaskPath+'workerstatus')):
            exit(1)
    """
    return True
    

    
    
    
    
    
    
    
    
