#coding=utf-8
import os,sys
path = os.path.abspath(__file__)
absdir = path[0:path.rfind('/')]
sys.path.append(absdir[0:absdir.rfind('/',0,len(absdir))])

import random,time
import threading
from common.const import type
from multiprocessing import cpu_count,Queue
from common.sdk import s3client,qiniuclient,aliyunclient,tencentclient
from utils import timeoperation
import savetolocal
import hashlib
import logging
from utils import readdata


log = logging.getLogger('root')

class Client():
    lock = threading.Lock()
    queue = None
    fileNumbersToCountDown = 0
    errorFileList = None
    md5List = None
    objectstatusFile = None
    isUploadAll = False
    #record
    succeeded = 0
    failed = 0
    failedSize = 0
    uploadSize = 0
    checkPartSize = 8*1024*1024
    def __init__(self, lines, taskInfo,objectstatusDict,probeResponseList,absdir):
        self.lines = lines
        task = taskInfo[0]
        self.taskID = taskInfo[1]
        [self.jobID,self.jobType,self.source,self.destination,self.keyForTaskPath,self.master,self.checkMode,self.sync] = task
        self.objectstatusDict = objectstatusDict
        self.probeResponseList = probeResponseList
        self.absdir = absdir
        self.fileType = self.source['filetype']
        if self.fileType == type.FileType.DiskFile:
            self.absolutePath = self.source['absolutepath']
        [self.desClient,self.srcClient] = self.create_clients()
        #self.cpus = cpu_count()
        self.cpus = 3
        line = self.lines.readline()
        pair = line.split('\t')
        self.fileNumbers = long(pair[0].strip())
        self.taskSize = long(pair[1].strip())
        self.initialClassVar()

    def initialClassVar(self):
        Client.queue = Queue(100)
        Client.fileNumbersToCountDown = long(self.fileNumbers)
        Client.errorFileList = list()
        Client.md5List = list()
        if self.jobType == type.JobType.Transfer:
            Client.objectstatusFile = savetolocal.create_objectstatusFile(self.absdir+'taskinfo/objectstatus')
        Client.isUploadAll = False
        #record
        Client.succeeded = 0
        Client.failed = 0
        Client.failedSize = 0
        Client.uploadSize = 0
        
    def create_clients(self):
        desClient = s3client.S3Client(self.destination['accessKey'],self.destination['secretKey'],self.destination['endpoint'])
        if self.fileType == type.FileType.QiniuFile:
            srcClient = qiniuclient.QiniuClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.fileType == type.FileType.S3File or self.fileType == type.FileType.BaiduFile:
            srcClient = s3client.S3Client(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.fileType == type.FileType.AliyunFile:
            srcClient = aliyunclient.AliyunClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        elif self.fileType == type.FileType.TencentFile:
            srcClient = tencentclient.TencentClient(self.source['accessKey'],self.source['secretKey'],self.source['endpoint'])
        else:
            srcClient = None
        return [desClient,srcClient]
    
    def transfer(self):
        [desClient,srcClient] = self.create_clients()
        while True:
            lineNum,line = self.get_from_queue()
            line = line.strip('\n').strip()
            pair = line.split('\t')
            key = pair[0].strip()
            size = pair[1].strip()
            if self.objectstatusDict:
                if self.objectstatusDict.has_key(str(lineNum)) and self.objectstatusDict[str(lineNum)] == '1':
                    log.info(str(key)+' is uploaded already')
                    self.save_objectstatus(key,lineNum,size,True)
                    continue
            isUploadOk = False
            recordErrorReason = ''
            try:
                [isRightTime, recordErrorReason] = self.isRightTimeToOpeareteThisFile(srcClient, key)
                if not isRightTime:
                    raise Exception
                if self.fileType == type.FileType.DiskFile: 
                    [isUploadOk,recordErrorReason] = self.upload_diskfile(desClient,key)
                else:
                    isOK,streamingBody = srcClient.get_object_with_StreamingBody(self.source['bucketName'],key)
                    if not isOK:
                        recordErrorReason = streamingBody
                        raise Exception
                    if self.fileType == type.FileType.QiniuFile:
                        [isUploadOk,recordErrorReason] = self.upload_qiufile(desClient, key,streamingBody)
                    elif (self.fileType == type.FileType.S3File 
                          or self.fileType == type.FileType.AliyunFile 
                          or self.fileType == type.FileType.TencentFile
                          or self.fileType == type.FileType.BaiduFile):
                        [isUploadOk,recordErrorReason] = self.upload_s3file(desClient, key, streamingBody)

            except Exception,e:
                log.error(e)
                isUploadOk = False
            finally:
                self.save_objectstatus(key, lineNum, size, isUploadOk, recordErrorReason)
                
    def isRightTimeToOpeareteThisFile(self, srcClient, key):
        if self.fileType == type.FileType.DiskFile:
            lastModify = os.path.getmtime(self.absolutePath+key)
        else:
            lastModify = srcClient.get_object_lastmodify(self.source['bucketName'], key)
        if lastModify == -1:
            return [False, key + ' not exists']
        else:
            if float(lastModify) >= float(self.sync['lastmodify']) and float(lastModify) <= float(self.sync['starttime']):
                return [True, '']
            else:
                return [False, timeoperation.fmt_time(lastModify)+' is not between '+
                        timeoperation.fmt_time(self.sync['lastmodify'])+' and '+
                        timeoperation.fmt_time(self.sync['starttime'])]
                
    def upload_diskfile(self,desClient,key):
        try:
            myfile = open(self.absolutePath+key,'r')
            mybool,res = desClient.upload_fileobj( self.destination['bucketName'], self.destination['key']+key, myfile)
            if not mybool:
                return [False,res]
            myfile.close()
            return [True,None]
        except IOError,e:
            log.error(e)
            return [False,e]
        except Exception,e:
            log.error(e)
            return [False,e]
        
    def upload_qiufile(self,desClient, key, streamingBody):
        return desClient.put_object_with_qiniu_response(self.destination['bucketName'], self.destination['key']+key, streamingBody)
    
    def upload_s3file(self,desClient, key,streamingBody):
        return desClient.upload_fileobj(self.destination['bucketName'], self.destination['key']+key, streamingBody)
    
    def check(self):
        [desClient,srcClient] = self.create_clients()
        while True:
            lineNum,line = self.get_from_queue()
            line = line.strip('\n').strip()
            pair = line.split('\t')
            key = pair[0].strip()
            size = pair[1].strip()
            isCheckOk = False
            recordErrorReason = ''
            try:
                [isRightTime, recordErrorReason] = self.isRightTimeToOpeareteThisFile(srcClient, key)
                if not isRightTime:
                    raise Exception
                if self.checkMode == 'md5':
                    [isCheckOk,recordErrorReason] = self.check_with_md5(srcClient, desClient, key)
                elif self.checkMode == 'head':
                    [isCheckOk,recordErrorReason] = self.check_with_head(desClient, key)
            except Exception,e:
                log.error(e)
                isCheckOk = False
            finally:
                self.save_objectstatus(key, lineNum, size, isCheckOk, recordErrorReason)
    
    def check_with_head(self,desClient, key):
        return [desClient.does_object_exists(self.destination['bucketName'], self.destination['key']+key),'']
    
    def check_with_md5(self,srcClient, desClient,key):
        if self.fileType == type.FileType.DiskFile:
            srcMd5 = self.compute_diskfile_md5(self.absolutePath+key)
        elif self.fileType == type.FileType.QiniuFile:
            srcMd5 = self.compute_qiniufile_md5(srcClient,self.source['bucketName'],key)
        elif (self.fileType == type.FileType.S3File 
              or self.fileType == type.FileType.AliyunFile 
              or self.fileType == type.FileType.TencentFile 
              or self.fileType == type.FileType.BaiduFile):
            srcMd5 = self.compute_s3file_md5(srcClient,self.source['bucketName'],key)
        if not srcMd5:
            if self.fileType == type.FileType.DiskFile:
                return [False,self.absolutePath+key+' doesn\'t exist in the source ']
            else:
                return [False,key+' doesn\'t exist in the source ']
            
        desMd5 = self.compute_s3file_md5(desClient,self.destination['bucketName'],self.destination['key']+key)

        if not desMd5:
            return [False,self.destination['key']+key+' doesn\'t exist in the destination']
        if srcMd5 == desMd5:
            return [True,desMd5]
        else:
            return [False,'md5 wrong,src:'+srcMd5+' des:'+desMd5+' ']
        
    def compute_s3file_md5(self,client,bucketName,key):
        try:
            stream = client.get_object_with_stream(bucketName, key)
            if not stream:
                return False
            myhash = hashlib.md5()
            mybytes = stream.read(Client.checkPartSize)
            while mybytes:
                myhash.update(mybytes)
                mybytes = stream.read(Client.checkPartSize)
            return myhash.hexdigest().lower()
        except Exception,e:
            log.error(e)
            return False
        
    def compute_qiniufile_md5(self,client,bucketName,key):
        try:
            [isSuccess,response] = client.get_object_with_StreamingBody(bucketName, key)
            if not isSuccess:
                return False
            myhash = hashlib.md5()
            for chunk in response.iter_content(Client.checkPartSize):
                myhash.update(chunk)
            return myhash.hexdigest().lower()
        except Exception,e:
            log.error(e)
            return False
        
    def compute_diskfile_md5(self,fileName):
        try:
            myhash = hashlib.md5()
            myfile = open(fileName,'r')
            mybytes = myfile.read(Client.checkPartSize)
            while mybytes:
                myhash.update(mybytes)
                mybytes = myfile.read(Client.checkPartSize)
            myfile.close()
            return myhash.hexdigest().lower()
        except Exception,e:
            log.error(e)
            return False
        
    def save_objectstatus(self,key,lineNum,size,isObjectOperationOk,errorReason=''):
        Client.lock.acquire()
        if isObjectOperationOk:
            Client.succeeded = Client.succeeded + 1
            Client.uploadSize = Client.uploadSize+long(size)
            if self.jobType == type.JobType.Transfer:
                Client.objectstatusFile.write(str(lineNum)+'\t'+'1\n')
                Client.objectstatusFile.flush()
                log.info('uploaded ok:'+key + '\t'+ str(Client.succeeded)+'/'+ str(self.fileNumbers))
            elif self.jobType == type.JobType.Check:
                Client.md5List.append(key+'\t'+errorReason)
                log.info('check ok:'+key +'\tmd5:'+errorReason+ '\t'+ str(Client.succeeded)+'/'+ str(self.fileNumbers))
        else:
            Client.failed = Client.failed + 1
            Client.failedSize = Client.failedSize+long(size)
            Client.errorFileList.append(key+'\t'+str(size)+'\t'+str(errorReason))
            if self.jobType == type.JobType.Transfer:
                Client.objectstatusFile.write(str(lineNum)+'\t'+'0\n')
                log.info('upload failed:'+key + '\t'+ str(Client.succeeded)+'/'+ str(self.fileNumbers))
            elif self.jobType == type.JobType.Check:
                log.info('check failed:'+key +'\tmd5:'+errorReason + '\t'+ str(Client.succeeded)+'/'+ str(self.fileNumbers))
            
        Client.fileNumbersToCountDown -= 1
        if self.jobType == type.JobType.Transfer:
            Client.objectstatusFile.flush()
        Client.lock.release()
                
    def get_from_queue(self):
        while True:
            try:
                lineNum,line = Client.queue.get(True,5)
                return [lineNum,line]
            except Exception,e:
                if Client.isUploadAll:
                    exit(0)
                else:
                    continue

    def execute(self):
        startTime = time.time()
        subThreads= self.generate_threads()
        line = self.lines.readline()
        cntLine = 0 
        while (line):
            Client.queue.put([cntLine,line])
            cntLine+=1
            line = self.lines.readline()
        while Client.fileNumbersToCountDown != 0:
            time.sleep(random.random()*2)
        Client.isUploadAll = True
        for i in range(self.cpus+1):
            subThreads[i].join()
        
        self.upload_list()
        if self.jobType == type.JobType.Transfer:
            Client.objectstatusFile.close()
        log.info('the cost time is '+ str(round(time.time()-startTime))+'s'+ ' and the average rate is '\
                  + str(round(float(Client.uploadSize)/(time.time()-startTime)/1024,2))+'KB/s')
        return [self.fileNumbers,\
                str(self.taskSize),\
                Client.succeeded,\
                Client.failed,\
                str(Client.uploadSize),\
                round(time.time()-startTime),\
                round(float(Client.uploadSize)/(time.time()-startTime)/1024,2)]
        
    def generate_threads(self):
        subThreads = list()
        for i in range(self.cpus):
            if self.jobType == type.JobType.Transfer:
                th = threading.Thread(target=self.transfer, args=())
            else:
                th = threading.Thread(target=self.check, args=())
            subThreads.append(th)
            th.setDaemon(True)
            th.start()
        probeResponseThread = threading.Thread(target=self.generate_probe_response, args=())
        subThreads.append(probeResponseThread)
        probeResponseThread.setDaemon(True)
        probeResponseThread.start()
        return subThreads
                
    def upload_list(self):
        lines = readdata.readlines(self.desClient, self.destination['bucketName'],self.keyForTaskPath+'taskstatus')
        if (not lines):
            exit(1)
        lines.readline()
        taskID = lines.readline()
        if taskID != self.taskID:
            log.error(taskID + ' and ' + self.taskID + ' are not the same!')
            exit(1)
        if Client.errorFileList:
            Client.errorFileList.insert(0, str(Client.failed)+'\t'+str(Client.failedSize))
            if not (self.desClient.put_object(self.destination['bucketName'], self.keyForTaskPath+'errorlist',\
                                              '\n'.join(Client.errorFileList))):
                exit(1)           
        if Client.md5List and self.jobType == type.JobType.Check:
            if not (self.desClient.put_object(self.destination['bucketName'], self.keyForTaskPath+'md5list',\
                                              '\n'.join(Client.md5List))):
                exit(1)
   
    def generate_probe_response(self):
        currentRate = 0
        currentTime = time.time()
        currentUploadSize = 0
        log.info( 'fileNumbers:'+str(self.fileNumbers)+'\ttaskSize:'+str(self.taskSize))
        while True:
            if Client.isUploadAll == True:
                return
            time.sleep(3)
            if Client.uploadSize-currentUploadSize == 0:
                continue
            currentRate = 0.7 * currentRate + \
                          0.3 * (Client.uploadSize-currentUploadSize)/(time.time()-currentTime)
            #probe
            probeString = 'files:'+str(Client.succeeded)+'/'+str(self.fileNumbers)+'　'+\
                          'size:'+str(Client.uploadSize)+'/'+str(self.taskSize)+'　'+\
                          'rate:'+str(round(currentRate/1024,2))+'KB/s|'
            if round(currentRate/1024,2) != 0:
                probeString += timeoperation.s_to_hms_string((self.taskSize-Client.uploadSize)/round(currentRate,2))+'\n'
            else:
                probeString += 'we can\'t estimate the left time to finish'+'\n'
            if len(self.probeResponseList) >=2:
                self.probeResponseList[1] = probeString
            else:
                self.probeResponseList.append(probeString)
            currentTime = time.time()
            currentUploadSize = Client.uploadSize






